function D = min_membrane(D,icells,species,Mspecies)
%MIN_MEMBRANE Create diffusion matrix for a membrane.
%   D = MIN_MEMBRANE(D,ICELLS,SPECIES,MSPECIES)
%   Set diffusion constants for species SPECIES to zero in the 
%   cells listed in ICELLS. MSPECIES is the total number of species.
%
%   Limited error-checking is performed.

% J. Cullhed 08-08-12

% Get dofs to remove.
if isa(icells,'logical')
  error('Expecting index vector.');
end
icells = icells(:);
ixremove = [];
for s = species
  ixremove = [ixremove; Mspecies*(icells-1)+s];
end

% Set diffusion.
[i,j,s] = find(D);
ixremove = [find(ismember(i,ixremove)); find(ismember(j,ixremove))];
i(ixremove) = [];
j(ixremove) = [];
s(ixremove) = [];

% Must recreate diffusion matrix.
Ndofs = size(D,1);
ixkeep = find(s > 0);
D = sparse(i(ixkeep),j(ixkeep),s(ixkeep),Ndofs,Ndofs);
d = full(sum(D,2));
D = D+sparse(1:Ndofs,1:Ndofs,-d);
