/* Propensity definition for the 'bistab' system. */

#include <stdlib.h>
#include <stdio.h>
/* mxMalloc, mxFree */
#include "mex.h" 
/* Type definition of the propensity functions. */
#include "propensities.h"

/* Rate constants */
const double N_A = 6.022e23;
const double k1 = 150.0, kd = 10.0, ka=1.2e8, k4 = 6.0;

/* Reaction propensities. */
double rFun1(const int *x, double vol, const double *data, int sd)
/* E_A -> E_A + A */
{
  return (double) k1*x[2];
}

double rFun2(const int *x, double vol, const double *data, int sd)
/*E_B -> E_B +B */
{
  return (double) k1*x[3];
}

double rFun3(const int *x, double vol, const double *data, int sd)
/*E_A + B -> E_AB */
{
  return (double) ka*x[1]*x[2]/(N_A*vol*1000.0);
}

double rFun4(const int *x, double vol, const double *data, int sd)
/*E_AB -> E_A +B */
{
  return (double) kd*x[4];
}

double rFun5(const int *x, double vol, const double *data, int sd)
/*E_B + A -> E_BA */
{
  return (double) ka*x[0]*x[3]/(N_A*vol*1000.0);
}

double rFun6(const int *x, double vol, const double *data, int sd)
/*E_BA -> E_B +A */
{
  return (double) kd*x[5];
}

double rFun7(const int *x, double vol, const double *data, int sd)
/*EAB + B -> EAB2 */
{
  return (double) ka*x[1]*x[4]/(N_A*vol*1000.0);
}

double rFun8(const int *x, double vol, const double *data, int sd)
/*EAB2 -> EAB +B */
{
  return (double) kd*x[6];
}

double rFun9(const int *x, double vol, const double *data, int sd)
/*EBA + A -> EBA2 */
{
  return (double) ka*x[0]*x[5]/(N_A*vol*1000.0);
}

double rFun10(const int *x, double vol, const double *data, int sd)
/*EBA2 -> EBA +A */
{
  return (double) kd*x[7];
}

double rFun11(const int *x, double vol, const double *data, int sd)
/*A -> EmptySet */
{
  return (double) k4*x[0];
}

double rFun12(const int *x, double vol, const double *data, int sd)
/*B -> EmptySet */
{
  return (double) k4*x[1];
}

PropensityFun *ALLOC_propensities(void)
{
  PropensityFun *ptr = MALLOC(sizeof(PropensityFun)*12);
  
  ptr[0]=rFun1;
  ptr[1]=rFun2;
  ptr[2]=rFun3;
  ptr[3]=rFun4;
  ptr[4]=rFun5;
  ptr[5]=rFun6;
  ptr[6]=rFun7;
  ptr[7]=rFun8;
  ptr[8]=rFun9;
  ptr[9]=rFun10;
  ptr[10]=rFun11;
  ptr[11]=rFun12;
  
  return ptr;
}

void FREE_propensities(PropensityFun* ptr)
{
  FREE(ptr);
}


